#!/bin/bash

#=========================================
# Author   : colben
#=========================================

set -euo pipefail
export LANG=en_US.UTF-8

[ 'x86_64' == "$(uname -m)" ] && ARCH='' || ARCH="-$(uname -m)"
ROOT_DIR="$(cd $(dirname $0) && pwd)"
IMAGE="harbor.colben.cn/general/greatsql$ARCH:8"
BUNDLE_FILE=

if [ -t 0 ]; then
    function Print { echo -e "\033[36;1m$(date +'[%F %T]')\033[32;1m $*\033[0m"; }
    function Warn { echo -e "\033[36;1m$(date +'[%F %T]')\033[33;1m $*\033[0m"; }
    function Error { echo -e "\033[36;1m$(date +'[%F %T]')\033[31;1m $*\033[0m"; exit 1; }
else
    function Print { echo -e "$(date +'[%F %T INFO]') $*"; }
    function Warn { echo -e "$(date +'[%F %T WARN]') $*"; }
    function Error { echo -e "$(date +'[%F %T ERROR]') $*"; exit 1; }
fi

function Quit {
    local exitCode=$?
    pkill -f '^python2 -m SimpleHTTPServer 10080$' || true
    [ 0 -ne $exitCode ] && Error Failed to build or push image!
    [ -z "${END:-}" ] && echo && Error Interrupted manually!
    Print Succeeded to build and push image.
}

function YesOrNo {
    Warn $*
    local sw=
    while :; do
        read -p '(Yes/No/Quit) ' -n1 sw
        [[ "$sw" =~ ^Y|y$ ]] && echo && return 0
        [[ "$sw" =~ ^N|n$ ]] && echo && return 1
        [[ "$sw" =~ ^Q|q$ ]] && echo && exit 0
        [ -n "$sw" ] && echo
    done
}

function Update {
    Warn Check files ...
    cd /release/RUNTIME
    if [ -z "$ARCH" ]; then
        BUNDLE_FILE=greatsql-8.0.32-26.1.el8.amd64.rpm-bundle.tar.xz
    elif [ '-aarch64' == "$ARCH" ]; then
        BUNDLE_FILE=greatsql-8.0.32-26.1.el8.arm64.rpm-bundle.tar.xz
    else
        Error Unknown arch: $ARCH!
    fi
    [ -e $BUNDLE_FILE ] || Error Not found $BUNDLE_FILE!
    python2 -m SimpleHTTPServer 10080 &>/dev/null &
}

function Build {
    local yn
    cd $ROOT_DIR
    docker images --format='{{.Repository}}:{{.Tag}}' | grep "^$IMAGE$" \
        && Warn Removing image $IMAGE ... \
        && docker rmi $IMAGE
    Warn Building image: $IMAGE ...
    docker build --force-rm --build-arg ARCH="$ARCH" --build-arg BUNDLE_FILE="$BUNDLE_FILE" -t $IMAGE -f Dockerfile-greatsql8 .
    YesOrNo Push image: $IMAGE? && docker push $IMAGE
}

function Main {
    trap Quit EXIT
    Update
    Build
    END=1
}

# Start here
Main

